# coding: utf-8

"""
Notehub API

The OpenAPI definition for the Notehub.io API.

The version of the OpenAPI document: 1.2.0
Contact: engineering@blues.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictFloat,
    StrictInt,
    StrictStr,
    field_validator,
)
from typing import Any, ClassVar, Dict, List, Optional, Union
from notehub_py.models.alert_data_inner import AlertDataInner
from notehub_py.models.alert_notifications_inner import AlertNotificationsInner
from typing import Optional, Set
from typing_extensions import Self


class Alert(BaseModel):
    """
    Alert
    """  # noqa: E501

    uid: Optional[StrictStr] = Field(default=None, description="Alert UID")
    monitor_uid: Optional[StrictStr] = Field(default=None, description="Monitor UID")
    monitor_name: Optional[StrictStr] = Field(default=None, description="Monitor Name")
    device_uid: Optional[StrictStr] = Field(default=None, description="Device UID")
    created_at: Optional[StrictInt] = Field(
        default=None, description="The time the alert was created"
    )
    value: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=None, description="The value that triggered the alert"
    )
    resolved: Optional[StrictBool] = Field(
        default=None, description="If true, the alert has been resolved"
    )
    version: Optional[StrictInt] = Field(
        default=None, description="The version of the alert"
    )
    alert_source: Optional[StrictStr] = Field(
        default=None, description="The source of the alert"
    )
    source: Optional[StrictStr] = Field(
        default=None, description="The UID of the source of the alert"
    )
    monitor_type: Optional[StrictStr] = Field(
        default=None, description="The type of monitor that triggered the alert"
    )
    field_name: Optional[StrictStr] = Field(
        default=None, description="The field name that triggered the alert"
    )
    data: Optional[List[AlertDataInner]] = None
    notifications: Optional[List[AlertNotificationsInner]] = None
    __properties: ClassVar[List[str]] = [
        "uid",
        "monitor_uid",
        "monitor_name",
        "device_uid",
        "created_at",
        "value",
        "resolved",
        "version",
        "alert_source",
        "source",
        "monitor_type",
        "field_name",
        "data",
        "notifications",
    ]

    @field_validator("alert_source")
    def alert_source_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(["app", "device"]):
            raise ValueError("must be one of enum values ('app', 'device')")
        return value

    @field_validator("monitor_type")
    def monitor_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(["event", "device"]):
            raise ValueError("must be one of enum values ('event', 'device')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Alert from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in data (list)
        _items = []
        if self.data:
            for _item in self.data:
                if _item:
                    _items.append(_item.to_dict())
            _dict["data"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in notifications (list)
        _items = []
        if self.notifications:
            for _item in self.notifications:
                if _item:
                    _items.append(_item.to_dict())
            _dict["notifications"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Alert from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "uid": obj.get("uid"),
                "monitor_uid": obj.get("monitor_uid"),
                "monitor_name": obj.get("monitor_name"),
                "device_uid": obj.get("device_uid"),
                "created_at": obj.get("created_at"),
                "value": obj.get("value"),
                "resolved": obj.get("resolved"),
                "version": obj.get("version"),
                "alert_source": obj.get("alert_source"),
                "source": obj.get("source"),
                "monitor_type": obj.get("monitor_type"),
                "field_name": obj.get("field_name"),
                "data": (
                    [AlertDataInner.from_dict(_item) for _item in obj["data"]]
                    if obj.get("data") is not None
                    else None
                ),
                "notifications": (
                    [
                        AlertNotificationsInner.from_dict(_item)
                        for _item in obj["notifications"]
                    ]
                    if obj.get("notifications") is not None
                    else None
                ),
            }
        )
        return _obj
